Some demon_log_* variables changed into logfile_* variables that are
[rsync/rsync.git] / flist.c
diff --git a/flist.c b/flist.c
index 56e742c..501a02e 100644 (file)
--- a/flist.c
+++ b/flist.c
@@ -1,29 +1,25 @@
 /*
-   Copyright (C) Andrew Tridgell 1996
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
-
-/** @file flist.c
- * Generate and receive file lists
+ * Generate and receive file lists.
  *
- * @sa http://lists.samba.org/pipermail/rsync/2000-June/002351.html
+ * Copyright (C) 1996 Andrew Tridgell
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006 Wayne Davison
  *
- **/
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "rsync.h"
 
@@ -153,61 +149,53 @@ static void list_file_entry(struct file_struct *f)
        }
 }
 
-/**
- * Stat either a symlink or its referent, depending on the settings of
- * copy_links, copy_unsafe_links, etc.
- *
- * @retval -1 on error
- *
- * @retval 0 for success
+/* Stat either a symlink or its referent, depending on the settings of
+ * copy_links, copy_unsafe_links, etc.  Returns -1 on error, 0 on success.
  *
- * @post If @p path is a symlink, then @p linkbuf (of size @c
- * MAXPATHLEN) contains the symlink target.
+ * If path is the name of a symlink, then the linkbuf buffer (which must hold
+ * MAXPATHLEN chars) will be set to the symlink's target string.
  *
- * @post @p buffer contains information about the link or the
- * referrent as appropriate, if they exist.
- **/
-static int readlink_stat(const char *path, STRUCT_STAT *buffer, char *linkbuf)
+ * The stat structure pointed to by stp will contain information about the
+ * link or the referent as appropriate, if they exist. */
+static int readlink_stat(const char *path, STRUCT_STAT *stp, char *linkbuf)
 {
 #ifdef SUPPORT_LINKS
-       if (copy_links)
-               return do_stat(path, buffer);
-       if (link_stat(path, buffer, copy_dirlinks) < 0)
+       if (link_stat(path, stp, copy_dirlinks) < 0)
                return -1;
-       if (S_ISLNK(buffer->st_mode)) {
-               int l = readlink((char *)path, linkbuf, MAXPATHLEN - 1);
-               if (l == -1)
+       if (S_ISLNK(stp->st_mode)) {
+               int llen = readlink(path, linkbuf, MAXPATHLEN - 1);
+               if (llen < 0)
                        return -1;
-               linkbuf[l] = 0;
+               linkbuf[llen] = '\0';
                if (copy_unsafe_links && unsafe_symlink(linkbuf, path)) {
                        if (verbose > 1) {
                                rprintf(FINFO,"copying unsafe symlink \"%s\" -> \"%s\"\n",
                                        path, linkbuf);
                        }
-                       return do_stat(path, buffer);
+                       return safe_stat(path, stp);
                }
        }
        return 0;
 #else
-       return do_stat(path, buffer);
+       return do_stat(path, stp);
 #endif
 }
 
-int link_stat(const char *path, STRUCT_STAT *buffer, int follow_dirlinks)
+int link_stat(const char *path, STRUCT_STAT *stp, int follow_dirlinks)
 {
 #ifdef SUPPORT_LINKS
        if (copy_links)
-               return do_stat(path, buffer);
-       if (do_lstat(path, buffer) < 0)
+               return safe_stat(path, stp);
+       if (do_lstat(path, stp) < 0)
                return -1;
-       if (follow_dirlinks && S_ISLNK(buffer->st_mode)) {
+       if (follow_dirlinks && S_ISLNK(stp->st_mode)) {
                STRUCT_STAT st;
-               if (do_stat(path, &st) == 0 && S_ISDIR(st.st_mode))
-                       *buffer = st;
+               if (safe_stat(path, &st) == 0 && S_ISDIR(st.st_mode))
+                       *stp = st;
        }
        return 0;
 #else
-       return do_stat(path, buffer);
+       return do_stat(path, stp);
 #endif
 }
 
@@ -542,7 +530,7 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
        clean_fname(thisname, 0);
 
        if (sanitize_paths)
-               sanitize_path(thisname, thisname, "", 0);
+               sanitize_path(thisname, thisname, "", 0, NULL);
 
        if ((basename = strrchr(thisname, '/')) != NULL) {
                dirname_len = ++basename - thisname; /* counts future '\0' */
@@ -667,8 +655,6 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
        if (linkname_len) {
                file->u.link = bp;
                read_sbuf(f, bp, linkname_len - 1);
-               if (sanitize_paths)
-                       sanitize_path(bp, bp, "", lastdir_depth);
                bp += linkname_len;
        }
 #endif
@@ -749,7 +735,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
        }
        clean_fname(thisname, 0);
        if (sanitize_paths)
-               sanitize_path(thisname, thisname, "", 0);
+               sanitize_path(thisname, thisname, "", 0, NULL);
 
        memset(sum, 0, SUM_LENGTH);
 
@@ -853,7 +839,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
                ? MD4_SUM_LENGTH : 0;
 
        alloc_len = file_struct_len + dirname_len + basename_len
-           + linkname_len + sum_len;
+                 + linkname_len + sum_len;
        if (flist)
                bp = pool_alloc(flist->file_pool, alloc_len, "make_file");
        else {
@@ -934,7 +920,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
                int save_mode = file->mode;
                file->mode = S_IFDIR; /* Find a directory with our name. */
                if (flist_find(the_file_list, file) >= 0
-                   && do_stat(thisname, &st2) == 0 && S_ISDIR(st2.st_mode)) {
+                   && safe_stat(thisname, &st2) == 0 && S_ISDIR(st2.st_mode)) {
                        file->modtime = st2.st_mtime;
                        file->length = st2.st_size;
                        file->mode = st2.st_mode;
@@ -1082,6 +1068,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 
        io_start_buffering_out();
        if (filesfrom_fd >= 0) {
+               if (sanitize_paths)
+                       die_on_unsafe_path(argv[0], 0);
                if (argv[0] && !push_dir(argv[0])) {
                        rsyserr(FERROR, errno, "push_dir %s failed",
                                full_fname(argv[0]));
@@ -1098,13 +1086,13 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                if (use_ff_fd) {
                        if (read_filesfrom_line(filesfrom_fd, fbuf) == 0)
                                break;
-                       sanitize_path(fbuf, fbuf, "", 0);
+                       sanitize_path(fbuf, fbuf, "", 0, NULL);
                } else {
                        if (argc-- == 0)
                                break;
                        strlcpy(fbuf, *argv++, MAXPATHLEN);
                        if (sanitize_paths)
-                               sanitize_path(fbuf, fbuf, "", 0);
+                               sanitize_path(fbuf, fbuf, "", 0, NULL);
                }
 
                len = strlen(fbuf);
@@ -1135,6 +1123,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                                   && (len == 1 || fbuf[len-2] == '/');
                }
 
+               if (sanitize_paths)
+                       die_on_unsafe_path(fbuf, 1);
                if (link_stat(fbuf, &st, copy_dirlinks) != 0) {
                        io_error |= IOERR_GENERAL;
                        rsyserr(FERROR, errno, "link_stat %s failed",